@@ -1,23 +1,28 @@
 from django.conf import settings
 from django.contrib.auth import get_user_model
-from django.db import models
+from django.db import models, transaction
+from django.db.models import Q
 from django.utils.translation import gettext as _
 from garpix_utils.string import get_random_string
 import string
 from datetime import datetime, timezone, timedelta
 
 from garpix_user.exceptions import NotConfirmedException
+from garpix_user.exceptions import IncorrectCodeException, NoTimeLeftException, WaitException, UserUnregisteredException
 
 
 class RestorePasswordMixin(models.Model):
+    class RESTORE_BY(models.TextChoices):
+        PHONE = ('phone', _('Phone number'))
+        EMAIL = ('email', _('Email'))
 
     restore_password_confirm_code = models.CharField(_('Restore password code'),
                                                      max_length=15, null=True, blank=True)
     is_restore_code_confirmed = models.BooleanField(_('Restore code confirmed'), blank=True, default=False)
     restore_date = models.DateTimeField(_('Restore code sent date'), null=True)
+    restore_by = models.CharField(choices=RESTORE_BY.choices, default=RESTORE_BY.EMAIL, max_length=5)
 
-    def _check_user_data(self, username):
-        from garpix_user.exceptions import UserUnregisteredException
+    def _check_and_get_user(self, username):
 
         User = get_user_model()
 
@@ -26,16 +31,18 @@ class RestorePasswordMixin(models.Model):
         field_name = '/'.join([User._meta.get_field(
             field).verbose_name.title().lower() for field in USERNAME_FIELDS]).rstrip('/')
 
-        if user := self.user:
-            for field in USERNAME_FIELDS:
-                if getattr(user, field) == username and getattr(user, f'is_{field}_confirmed', False):
-                    return True, field
+        user_data = Q()
+
+        for field in USERNAME_FIELDS:
+            user_data |= Q(**{field: username, f'is_{field}_confirmed': True})
+
+        if user := User.objects.filter(user_data).first():
+            return True, user
 
         return False, UserUnregisteredException(field='username',
                                                 extra_data={'field': field_name})
 
     def _check_request_time(self):
-        from garpix_user.exceptions import WaitException
 
         if (TIME_LAST_REQUEST := settings.GARPIX_USER.get('TIME_LAST_REQUEST', 1)) and (
                 restore_date := self.restore_date):
@@ -44,12 +51,22 @@ class RestorePasswordMixin(models.Model):
 
         return True, None
 
+    def _time_is_up(self):
+        datediff = datetime.now(self.restore_date.tzinfo) - self.restore_date
+
+        if self.restore_by == self.RESTORE_BY.EMAIL:
+            return datediff.days > settings.GARPIX_USER.get(
+                'CONFIRM_EMAIL_CODE_LIFE_TIME', 6)
+        return datediff.seconds / 60 > settings.GARPIX_USER.get(
+            'CONFIRM_PHONE_CODE_LIFE_TIME', 6)
+
     def send_restore_code(self, username=None):
         from garpix_notify.models import Notify
 
-        result, data = self._check_user_data(username)
+        result, data = self._check_and_get_user(username)
         if not result:
             return result, data
+        user = data
         result, error = self._check_request_time()
         if not result:
             return result, error
@@ -58,32 +75,31 @@ class RestorePasswordMixin(models.Model):
 
         self.restore_password_confirm_code = confirmation_code
         self.restore_date = datetime.now(timezone.utc)
-        self.save()
+        self.is_restore_code_confirmed = False
 
-        if data == 'email' and 'email' in self.user.USERNAME_FIELDS:
+        if user.email == username and 'email' in user.USERNAME_FIELDS:
             Notify.send(settings.RESTORE_PASSWORD_EMAIL_EVENT, {
-                'user_fullname': str(self.user),
-                'email': self.user.email,
+                'user': user,
                 'restore_code': self.restore_password_confirm_code
-            }, email=self.user.email)
-        elif 'phone' in self.user.USERNAME_FIELDS:
+            }, email=user.email)
+            self.restore_by = self.RESTORE_BY.EMAIL
+        elif 'phone' in user.USERNAME_FIELDS:
             Notify.send(settings.RESTORE_PASSWORD_PHONE_EVENT, {
-                'user_fullname': str(self.user),
-                'phone': self.user.phone,
-                'restore_code': self.user.restore_password_confirm_code
-            }, phone=self.user.phone)
+                'user': user,
+                'restore_code': self.restore_password_confirm_code
+            }, phone=user.phone)
+            self.restore_by = self.RESTORE_BY.PHONE
+
+        self.save()
 
         return True, None
 
     def check_restore_code(self, restore_password_confirm_code=None):
-        from garpix_user.exceptions import IncorrectCodeException, NoTimeLeftException
 
         if self.restore_password_confirm_code != restore_password_confirm_code:
             return False, IncorrectCodeException(field='restore_password_confirm_code')
 
-        time_is_up = (datetime.now(
-            self.restore_date.tzinfo) - self.restore_date).seconds / 60 > settings.GARPIX_USER.get(
-            'CONFIRM_EMAIL_CODE_LIFE_TIME', 6)
+        time_is_up = self._time_is_up()
 
         if time_is_up:
             return False, NoTimeLeftException(field='restore_password_confirm_code')
@@ -93,13 +109,34 @@ class RestorePasswordMixin(models.Model):
 
         return True, None
 
-    def restore_password(self, field, new_password=None):
+    def restore_password(self, new_password, username):
+        User = get_user_model()
+
+        USERNAME_FIELDS = getattr(User, 'USERNAME_FIELDS', ('email',))
+
+        field_name = '/'.join([User._meta.get_field(
+            field).verbose_name.title().lower() for field in USERNAME_FIELDS]).rstrip('/')
+
         if self.is_restore_code_confirmed:
-            self.user.set_password(new_password)
-            self.user.save()
+
+            time_is_up = self._time_is_up()
+
+            if time_is_up:
+                return False, NoTimeLeftException(field='restore_password_confirm_code')
+
+            result, data = self._check_and_get_user(username)
+            if not result:
+                return result, data
+
+            with transaction.atomic():
+                user = data
+                user.set_password(new_password)
+                user.save()
+                self.is_restore_code_confirmed = False
+                self.save()
             return True, None
         return False, NotConfirmedException(
-            extra_data={'username': self._meta.get_field(field).verbose_name.title().lower()})
+            extra_data={'field': field_name})
 
     class Meta:
         abstract = True
